﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Controls.Primitives;
using Infragistics.Controls.Editors;
using System.Collections;
using Infragistics.Controls.Editors.Primitives;

namespace xamMultiColumnCombo
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindowViewModel ViewModel { get; set; }

        public MainWindow()
        {
            this.DataContext = (ViewModel = new MainWindowViewModel());

            InitializeComponent();
        }

        private void xamMultiColumnComboEditor1_Loaded(object sender, RoutedEventArgs e)
        {
            Grid grid = VisualTreeHelper.GetChild(sender as XamMultiColumnComboEditor, 0) as Grid;
            Popup popup = ((grid.Children[0] as Border).Child as Grid).Children[0] as Popup;

            // Handle the popup opened event.  The grid rows will exist by the time this event is fired.
            popup.Opened += new EventHandler(popup_Opened);
        }

        void popup_Opened(object sender, EventArgs e)
        {
            Popup popup = sender as Popup;
            Border popupBorder = (popup.Child as System.Windows.Controls.Grid).Children[0] as Border;

            // This items panel contains all the rows, including the header, in the grid.
            MultiColumnComboItemsPanel itemsPanel = FindItemsPanel(popup.Child) as MultiColumnComboItemsPanel;
            if (itemsPanel != null && itemsPanel.Children.Count != 0)
            {
                double verticalScrollBarWidth = ((itemsPanel.Parent as Grid).Children[1] as FrameworkElement).Width;
                double horizontalScrollBarHeight = ((itemsPanel.Parent as Grid).Children[2] as FrameworkElement).Height;

                // Header panel.
                ComboCellsPanel headerPanel = itemsPanel.Children[0] as ComboCellsPanel;

                // First panel with actual data in it.
                ComboCellsPanel actualRowPanel = itemsPanel.Children[1] as ComboCellsPanel;

                double headerHeight = headerPanel.ActualHeight;
                double actualRowHeight = actualRowPanel.ActualHeight * ViewModel.Data.Count;
                double actualRowWidth = 0;
                for (int i = 0; i < actualRowPanel.Children.Count; i++)
                {
                    ComboCellControl control = actualRowPanel.Children[i] as ComboCellControl;
                    actualRowWidth += control.ActualWidth;
                }

                // The 5 is just to push the width over enough to hide the scrollbar.
                popupBorder.Width = actualRowWidth + 5;
                popupBorder.Height = headerHeight + actualRowHeight;
            }
        }

        private void xamMultiColumnComboEditor1_DropDownOpened(object sender, EventArgs e)
        {
            Grid grid = VisualTreeHelper.GetChild(sender as XamMultiColumnComboEditor, 0) as Grid;
            Popup popup = ((grid.Children[0] as Border).Child as Grid).Children[0] as Popup;

            ((popup.Child as System.Windows.Controls.Grid).Children[0] as Border).Width = double.MaxValue;
        }

        /// <summary>
        /// Helper method used to find the first grid row in the visual tree.
        /// </summary>
        /// <param name="root"></param>
        /// <returns></returns>
        private DependencyObject FindItemsPanel(DependencyObject root)
        {
            for (int i = 0; i < VisualTreeHelper.GetChildrenCount(root); i++)
            {
                DependencyObject dObj = VisualTreeHelper.GetChild(root, i);
                if (dObj.GetType() == typeof(MultiColumnComboItemsPanel))
                    return dObj;

                dObj = FindItemsPanel(dObj);
                if (dObj != null)
                    return dObj;
            }

            return null;
        }
    }
}
